-----------------------------------------------
-- Licensed Materials - Property of IBM
--
-- 5724-R31, 5655-S30
--
-- (C) Copyright IBM Corp. 2006, 2008 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
------------------------------------------------

CREATE TABLE @SCHEMA@.w_lit_double (
    id INTEGER NOT NULL, 
    litval DOUBLE);

ALTER TABLE @SCHEMA@.w_lit_double ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_dbl ON @SCHEMA@.w_lit_double (litval);

CREATE TABLE @SCHEMA@.w_lit_float (
    id INTEGER NOT NULL, 
    litval REAL ); 

ALTER TABLE @SCHEMA@.w_lit_float ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_flt ON @SCHEMA@.w_lit_float (litval);

CREATE TABLE @SCHEMA@.w_lit_long (
    id INTEGER NOT NULL, 
    litval BIGINT ); 

ALTER TABLE @SCHEMA@.w_lit_long ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_long ON @SCHEMA@.w_lit_long (litval);


CREATE TABLE @SCHEMA@.w_obj_lit_any (
    id INTEGER NOT NULL, 
    large LONG VARCHAR FOR BIT DATA, 
    "HASH" VARCHAR(40), 
    type_uri VARCHAR(254) NOT NULL ); 

ALTER TABLE @SCHEMA@.w_obj_lit_any ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_any ON @SCHEMA@.w_obj_lit_any (type_uri);


CREATE TABLE @SCHEMA@.w_obj_lit_date (
    id INTEGER NOT NULL, 
    litval TIMESTAMP ); 

ALTER TABLE @SCHEMA@.w_obj_lit_date ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_date ON @SCHEMA@.w_obj_lit_date (litval);


CREATE TABLE @SCHEMA@.w_obj_lit_datetime (
    id INTEGER NOT NULL, 
    litval TIMESTAMP ); 


ALTER TABLE @SCHEMA@.w_obj_lit_datetime ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_time ON @SCHEMA@.w_obj_lit_datetime (litval);


CREATE TABLE @SCHEMA@.w_obj_lit_string (
    id INTEGER NOT NULL,
    large LONG VARCHAR FOR BIT DATA,
    litval VARCHAR(1024) CCSID 1208,
    hash VARCHAR(40) );

ALTER TABLE @SCHEMA@.w_obj_lit_string ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_obj_str ON @SCHEMA@.w_obj_lit_string (hash);


CREATE TABLE @SCHEMA@.w_namespace (
    id INTEGER NOT NULL, 
    namespace VARCHAR(254) NOT NULL ); 

ALTER TABLE @SCHEMA@.w_namespace ADD PRIMARY KEY (id);

CREATE UNIQUE INDEX @SCHEMA@.idx_namespace ON @SCHEMA@.w_namespace (namespace ASC);


CREATE TABLE @SCHEMA@.w_uri (
    id INTEGER NOT NULL, 
    uri VARCHAR(254) NOT NULL, 
    namespace_id INTEGER NOT NULL ); 

ALTER TABLE @SCHEMA@.w_uri ADD PRIMARY KEY (id);

CREATE UNIQUE INDEX @SCHEMA@.idx_uri ON @SCHEMA@.w_uri (uri ASC);
CREATE UNIQUE INDEX @SCHEMA@.idx_uri_by_ns ON @SCHEMA@.w_uri (namespace_id,id);


CREATE TABLE @SCHEMA@.w_statement (
    id INTEGER NOT NULL, 
    version_from INTEGER NOT NULL, 
    version_to INTEGER NOT NULL, 
    subj_id INTEGER NOT NULL, 
    pred_id INTEGER NOT NULL, 
    obj_id INTEGER NOT NULL, 
    obj_typ_cd INTEGER NOT NULL,
    partition_id INTEGER NOT NULL); 

ALTER TABLE @SCHEMA@.w_statement ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_smt_by_sbj ON @SCHEMA@.w_statement (subj_id, version_from, version_to);
CREATE INDEX @SCHEMA@.idx_smt_by_fvr ON @SCHEMA@.w_statement (version_from);
CREATE INDEX @SCHEMA@.idx_sbj_by_prp ON @SCHEMA@.w_statement (pred_id, obj_id);
CREATE INDEX @SCHEMA@.idx_sbj_by_val ON @SCHEMA@.w_statement (obj_id, subj_id);
CREATE INDEX @SCHEMA@.idx_val_by_prp ON @SCHEMA@.w_statement (subj_id, pred_id);


CREATE TABLE @SCHEMA@.w_version (
    username VARCHAR(64) NOT NULL, 
    change_time TIMESTAMP NOT NULL, 
    cl_gid VARCHAR(36), 
    cl_lid VARCHAR(10), 
    schema_ns_id INTEGER, 
    schema_rev INTEGER, 
    id INTEGER NOT NULL,
    partition_id INTEGER NOT NULL ); 

ALTER TABLE @SCHEMA@.w_version ADD PRIMARY KEY (id, partition_id);

CREATE INDEX @SCHEMA@.idx_ver_schema ON @SCHEMA@.w_version (schema_ns_id);

CREATE TABLE @SCHEMA@.w_artifact_blob (
 id VARCHAR(254) NOT NULL, 
 content BLOB(1G), 
 deleted SMALLINT,
 PRIMARY KEY(id)); 

CREATE TABLE @SCHEMA@.w_locale (
 id INTEGER NOT NULL,
 locale VARCHAR(8) NOT NULL );

ALTER TABLE @SCHEMA@.w_locale ADD PRIMARY KEY (id);

ALTER TABLE @SCHEMA@.w_locale ADD CONSTRAINT @SCHEMA@.unique_locale UNIQUE (locale);

INSERT INTO @SCHEMA@.w_locale VALUES (1000, '');
INSERT INTO @SCHEMA@.w_locale VALUES (1001, 'en');
INSERT INTO @SCHEMA@.w_locale VALUES (1002, 'en-us');
INSERT INTO @SCHEMA@.w_locale VALUES (1003, 'en-uk');
INSERT INTO @SCHEMA@.w_locale VALUES (1004, 'en-gb');
INSERT INTO @SCHEMA@.w_locale VALUES (1032, 'cs');
INSERT INTO @SCHEMA@.w_locale VALUES (1064, 'es');
INSERT INTO @SCHEMA@.w_locale VALUES (1096, 'de');
INSERT INTO @SCHEMA@.w_locale VALUES (1128, 'de-de');
INSERT INTO @SCHEMA@.w_locale VALUES (1129, 'fr');
INSERT INTO @SCHEMA@.w_locale VALUES (1160, 'fr-ca-sk');
INSERT INTO @SCHEMA@.w_locale VALUES (1161, 'hu');
INSERT INTO @SCHEMA@.w_locale VALUES (1192, 'it');
INSERT INTO @SCHEMA@.w_locale VALUES (1224, 'ja');
INSERT INTO @SCHEMA@.w_locale VALUES (1256, 'ko');
INSERT INTO @SCHEMA@.w_locale VALUES (1288, 'pl');
INSERT INTO @SCHEMA@.w_locale VALUES (1320, 'pt-br');
INSERT INTO @SCHEMA@.w_locale VALUES (1352, 'ru');
INSERT INTO @SCHEMA@.w_locale VALUES (1384, 'zh');
INSERT INTO @SCHEMA@.w_locale VALUES (1416, 'zh-tw');
INSERT INTO @SCHEMA@.w_locale VALUES (1448, 'el');

		
---------------------------------------------------------------------------------
-----Create Foreign keys on trplestore tables
---------------------------------------------------------------------------------

ALTER TABLE @SCHEMA@.w_uri 
	ADD CONSTRAINT @SCHEMA@.w_uri_fk_namesp FOREIGN KEY
		(namespace_id)
	REFERENCES @SCHEMA@.w_namespace
		(id)
	ON DELETE NO ACTION
	ON UPDATE NO ACTION;


ALTER TABLE @SCHEMA@.w_version 
	ADD CONSTRAINT @SCHEMA@.w_ver_fk_namesp FOREIGN KEY
		(schema_ns_id)
	REFERENCES @SCHEMA@.w_namespace
		(id)
	ON DELETE NO ACTION
	ON UPDATE NO ACTION;


CREATE SEQUENCE @SCHEMA@.seq_w_lit_double_id 
 AS INTEGER START WITH 0 INCREMENT BY 1
 NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_lit_float_id 
AS INTEGER START WITH 0 INCREMENT BY 1
 NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_lit_long_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_namespace_id
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_obj_lit_any_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_obj_lit_date_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_obj_lit_datetime_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_obj_lit_string_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_statement_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;

CREATE SEQUENCE @SCHEMA@.seq_w_uri_id 
AS INTEGER START WITH 0 INCREMENT BY 1
NO MAXVALUE MINVALUE 1
NO CYCLE CACHE 20 ORDER;



-- Pattern tables for query optimization

CREATE TABLE @SCHEMA@.w_subj_pred_index (
    subj_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    predicate_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (subj_id, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_subj_obj_index (
    subj_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    object_map VARCHAR (1024) FOR BIT DATA,
    obj_typ_cd INTEGER NOT NULL,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (subj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_pred_subj_index (
    pred_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    subject_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (pred_id, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_pred_obj_index (
    pred_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    object_map VARCHAR (1024) FOR BIT DATA,
    obj_typ_cd INTEGER NOT NULL,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (pred_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_obj_subj_index (
    obj_id INTEGER NOT NULL,
    obj_typ_cd INTEGER NOT NULL,
    subject_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_obj_pred_index (
    obj_id INTEGER NOT NULL,
    obj_typ_cd INTEGER NOT NULL,
    predicate_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);


CREATE TABLE @SCHEMA@.w_dbversion (
    subsystem VARCHAR(8) NOT NULL, 
    cur_version INTEGER NOT NULL );

ALTER TABLE @SCHEMA@.w_dbversion ADD PRIMARY KEY (subsystem);
