
--
--  Create temporary tables, copy the original data into them and drop the original tables
--      This is just to make sure all of the indexes on the original table are gone.
--

CREATE TABLE @SCHEMA@.w_temp_uri (
    id INTEGER NOT NULL, 
    uri VARCHAR(254) NOT NULL, 
    namespace_id INTEGER NOT NULL ); 

INSERT INTO @SCHEMA@.w_temp_uri SELECT * FROM @SCHEMA@.w_uri;
DROP TABLE @SCHEMA@.w_uri;

CREATE TABLE @SCHEMA@.w_temp_statement (
    id INTEGER NOT NULL, 
    version_from INTEGER NOT NULL, 
    version_to INTEGER NOT NULL, 
    subj_id INTEGER NOT NULL, 
    pred_id INTEGER NOT NULL, 
    obj_id INTEGER NOT NULL, 
    obj_typ_cd INTEGER NOT NULL,
    partition_id INTEGER NOT NULL); 


INSERT INTO @SCHEMA@.w_temp_statement SELECT * FROM @SCHEMA@.w_statement;
DROP TABLE @SCHEMA@.w_statement;

--
-- Recreate the tables with the new indexes
--

CREATE TABLE @SCHEMA@.w_uri (
    id INTEGER NOT NULL, 
    uri VARCHAR(254) NOT NULL, 
    namespace_id INTEGER NOT NULL ); 

ALTER TABLE @SCHEMA@.w_uri ADD PRIMARY KEY (id);

CREATE UNIQUE INDEX @SCHEMA@.idx_uri ON @SCHEMA@.w_uri (uri ASC);
CREATE UNIQUE INDEX @SCHEMA@.idx_uri_by_ns ON @SCHEMA@.w_uri (namespace_id,id);

ALTER TABLE @SCHEMA@.w_uri 
	ADD CONSTRAINT @SCHEMA@.w_uri_fk_namesp FOREIGN KEY
		(namespace_id)
	REFERENCES @SCHEMA@.w_namespace
		(id)
	ON DELETE NO ACTION
	ON UPDATE NO ACTION;



CREATE TABLE @SCHEMA@.w_statement (
    id INTEGER NOT NULL, 
    version_from INTEGER NOT NULL, 
    version_to INTEGER NOT NULL, 
    subj_id INTEGER NOT NULL, 
    pred_id INTEGER NOT NULL, 
    obj_id INTEGER NOT NULL, 
    obj_typ_cd INTEGER NOT NULL,
    partition_id INTEGER NOT NULL); 

ALTER TABLE @SCHEMA@.w_statement ADD PRIMARY KEY (id);

CREATE INDEX @SCHEMA@.idx_smt_by_sbj ON @SCHEMA@.w_statement (subj_id, version_from, version_to);
CREATE INDEX @SCHEMA@.idx_smt_by_fvr ON @SCHEMA@.w_statement (version_from);
CREATE INDEX @SCHEMA@.idx_sbj_by_prp ON @SCHEMA@.w_statement (pred_id, obj_id);
CREATE INDEX @SCHEMA@.idx_sbj_by_val ON @SCHEMA@.w_statement (obj_id, subj_id);
CREATE INDEX @SCHEMA@.idx_val_by_prp ON @SCHEMA@.w_statement (subj_id, pred_id);

--
--  Copy the data from the temporary tables back into the original tables and drop the temporary tables
--

INSERT INTO @SCHEMA@.w_statement SELECT * FROM @SCHEMA@.w_temp_statement;
DROP TABLE @SCHEMA@.w_temp_statement;

INSERT INTO @SCHEMA@.w_uri SELECT * FROM @SCHEMA@.w_temp_uri;
DROP TABLE @SCHEMA@.w_temp_uri;

-----------------------------------------------
-- Licensed Materials - Property of IBM
--
-- 5724-R31, 5655-S30
--
-- (C) Copyright IBM Corp. 2009 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
------------------------------------------------

CREATE TABLE @SCHEMA@.w_locale (
    id INTEGER NOT NULL, 
    locale VARCHAR(8) NOT NULL);
 
ALTER TABLE @SCHEMA@.w_locale ADD PRIMARY KEY (id);

ALTER TABLE @SCHEMA@.w_locale ADD CONSTRAINT @SCHEMA@.unique_locale UNIQUE(locale);

INSERT INTO @SCHEMA@.w_locale VALUES (1000, '');
INSERT INTO @SCHEMA@.w_locale VALUES (1001, 'en');
INSERT INTO @SCHEMA@.w_locale VALUES (1002, 'en-us');
INSERT INTO @SCHEMA@.w_locale VALUES (1003, 'en-uk');
INSERT INTO @SCHEMA@.w_locale VALUES (1004, 'en-gb');
INSERT INTO @SCHEMA@.w_locale VALUES (1032, 'cs');
INSERT INTO @SCHEMA@.w_locale VALUES (1064, 'es');
INSERT INTO @SCHEMA@.w_locale VALUES (1096, 'de');
INSERT INTO @SCHEMA@.w_locale VALUES (1128, 'de-de');
INSERT INTO @SCHEMA@.w_locale VALUES (1129, 'fr');
INSERT INTO @SCHEMA@.w_locale VALUES (1160, 'fr-ca-sk');
INSERT INTO @SCHEMA@.w_locale VALUES (1161, 'hu');
INSERT INTO @SCHEMA@.w_locale VALUES (1192, 'it');
INSERT INTO @SCHEMA@.w_locale VALUES (1224, 'ja');
INSERT INTO @SCHEMA@.w_locale VALUES (1256, 'ko');
INSERT INTO @SCHEMA@.w_locale VALUES (1288, 'pl');
INSERT INTO @SCHEMA@.w_locale VALUES (1320, 'pt-br');
INSERT INTO @SCHEMA@.w_locale VALUES (1352, 'ru');
INSERT INTO @SCHEMA@.w_locale VALUES (1384, 'zh');
INSERT INTO @SCHEMA@.w_locale VALUES (1416, 'zh-tw');
INSERT INTO @SCHEMA@.w_locale VALUES (1448, 'el');

-- Commit the active changeset which prevents the bootstrap of additional 
-- content either during migration or after installing stack product such as Fabric.
UPDATE @SCHEMA@.w_statement 
        SET obj_id=(SELECT id FROM @SCHEMA@.w_obj_lit_string WHERE litval='PUBLISHED') 
	WHERE pred_id=(SELECT id FROM @SCHEMA@.w_uri WHERE uri LIKE '%changeSetState')
	AND obj_id IN (SELECT id FROM @SCHEMA@.w_obj_lit_string WHERE litval IN ('DRAFT', 'PENDING' , 'APPROVED'))
	AND obj_typ_cd=6
	AND version_to=2000000000;


-- Can't drop plain until contents copied into 
-- @SCHEMA@.w_string, with lang in @SCHEMA@.w_statement.

UPDATE @SCHEMA@.w_statement SET obj_id 
 = (SELECT @SCHEMA@.w_lit_bool.litval FROM @SCHEMA@.w_lit_bool
     WHERE @SCHEMA@.w_statement.obj_id = @SCHEMA@.w_lit_bool.id)
WHERE @SCHEMA@.w_statement.obj_typ_cd = 7;

UPDATE @SCHEMA@.w_statement SET obj_id 
 = (SELECT @SCHEMA@.w_lit_int.litval FROM @SCHEMA@.w_lit_int
     WHERE @SCHEMA@.w_statement.obj_id = @SCHEMA@.w_lit_int.id)
WHERE @SCHEMA@.w_statement.obj_typ_cd = 2;

-- Merging string and plain:
-- 1. Canonicalize locale.

UPDATE @SCHEMA@.w_obj_lit_plain SET xml_lang = '' WHERE xml_lang is null;


-- Guarantee no ID collisions during the copy:

UPDATE @SCHEMA@.w_statement
   SET obj_id = (obj_id + 1 + (SELECT max(id) from @SCHEMA@.w_obj_lit_string))
 WHERE obj_typ_cd = 11;

UPDATE @SCHEMA@.w_obj_lit_plain 
  SET id = (id + 1 + (SELECT max(@SCHEMA@.w_obj_lit_string.id) FROM @SCHEMA@.w_obj_lit_string));

-- 3. Copy from plain into string table, with null  hash.

INSERT INTO @SCHEMA@.w_obj_lit_string (id, large, litval) 
  SELECT id, large, litval
    FROM @SCHEMA@.w_obj_lit_plain;

-- Use locale table to get type codes for plains.
UPDATE @SCHEMA@.w_statement SET obj_typ_cd 
  = (SELECT @SCHEMA@.w_locale.id FROM @SCHEMA@.w_locale
      WHERE @SCHEMA@.w_locale.locale =
              (SELECT @SCHEMA@.w_obj_lit_plain.xml_lang 
                 FROM @SCHEMA@.w_obj_lit_plain 
                WHERE @SCHEMA@.w_statement.obj_id = @SCHEMA@.w_obj_lit_plain.id))
WHERE @SCHEMA@.w_statement.obj_typ_cd = 11;

-- The old object IDs in the string table haven't changed, but
-- the type code has. 
UPDATE @SCHEMA@.w_statement SET obj_typ_cd = 999 WHERE obj_typ_cd = 6;

-- Cleanup

DROP TABLE @SCHEMA@.w_obj_lit_plain;
DROP TABLE @SCHEMA@.w_lit_int;
DROP TABLE @SCHEMA@.w_lit_bool;

DROP SEQUENCE @SCHEMA@.seq_w_obj_lit_plain_id;
DROP SEQUENCE @SCHEMA@.seq_w_lit_int_id;
DROP SEQUENCE @SCHEMA@.seq_w_lit_bool_id;



-- Pattern tables for query optimization

CREATE TABLE @SCHEMA@.w_subj_pred_index (
    subj_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    predicate_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (subj_id, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_subj_obj_index (
    subj_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    object_map VARCHAR (1024) FOR BIT DATA,
    obj_typ_cd INTEGER NOT NULL,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (subj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_pred_subj_index (
    pred_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    subject_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (pred_id, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_pred_obj_index (
    pred_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    object_map VARCHAR (1024) FOR BIT DATA,
    obj_typ_cd INTEGER NOT NULL,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (pred_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_obj_subj_index (
    obj_id INTEGER NOT NULL,
    obj_typ_cd INTEGER NOT NULL,
    subject_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);

CREATE TABLE @SCHEMA@.w_obj_pred_index (
    obj_id INTEGER NOT NULL,
    obj_typ_cd INTEGER NOT NULL,
    predicate_map VARCHAR (1024) FOR BIT DATA,
    version_from INTEGER NOT NULL,
    version_to INTEGER NOT NULL,
    partition_id INTEGER NOT NULL REFERENCES @SCHEMA@.w_uri (id),
    PRIMARY KEY (obj_id, obj_typ_cd, version_from, version_to, partition_id)
);

